﻿using System;
using System.Collections.Generic;
using System.Linq;

using Foundation;
using UIKit;
using Syncfusion.SfChart.XForms.iOS.Renderers;
using Syncfusion.SfChart.iOS;
using CoreGraphics;
using Syncfusion.SfChart.XForms;
using Xamarin.Forms;

namespace ChartSample.iOS
{
    // The UIApplicationDelegate for the application. This class is responsible for launching the 
    // User Interface of the application, as well as listening (and optionally responding) to 
    // application events from iOS.
    [Register("AppDelegate")]
    public partial class AppDelegate : global::Xamarin.Forms.Platform.iOS.FormsApplicationDelegate
    {
        //
        // This method is invoked when the application has loaded and is ready to run. In this 
        // method you should instantiate the window, load the UI into it and then make the window
        // visible.
        //
        // You have 17 seconds to return from this method, or iOS will terminate your application.
        //
        public override bool FinishedLaunching(UIApplication app, NSDictionary options)
        {
            global::Xamarin.Forms.Forms.Init();
            new SfChartRenderer();

            // Code for starting up the Xamarin Test Cloud Agent
#if ENABLE_TEST_CLOUD
			Xamarin.Calabash.Start();
#endif

            LoadApplication(new App());

            Xamarin.Forms.Forms.ViewInitialized += (sender, e) =>
            {
                SFChart chart = (e.NativeView as SFChart);

                if (chart != null)
                {
                    foreach (var gesture in chart.GestureRecognizers)
                    {
                        if (gesture is UIPanGestureRecognizer && gesture.Delegate == null)
                        {
                            gesture.Delegate = new CustomGeatureDelegate(e.View as SfChart);
                        }
                    }
                }
            };

            return base.FinishedLaunching(app, options);
        }
    }

    public class CustomGeatureDelegate : UIGestureRecognizerDelegate
    {
        SfChart formChart;

        public CustomGeatureDelegate(SfChart chart)
        {
            formChart = chart;
        }

        public override bool ShouldBegin(UIGestureRecognizer recognizer)
        {
            if (recognizer is UIPanGestureRecognizer)
            {
                SFChart chart = recognizer.View as SFChart;

                CGPoint velocity = (recognizer as UIPanGestureRecognizer).VelocityInView(recognizer.View);

                if (Math.Abs(velocity.Y) < Math.Abs(velocity.X))
                {
                    if (chart.PrimaryAxis != null)
                    {
                        float positon = chart.PrimaryAxis.ZoomPosition;
                        float factor = 1.0f - chart.PrimaryAxis.ZoomFactor;

                        if ((positon == factor && velocity.X < 0) || (positon == 0 && velocity.X > 0))
                        {
                            bool isPanEndRange = false;
                            for (int i = 0; i < formChart.Series.Count; i++)
                            {
                                SFCartesianSeries series = chart.SeriesAtIndex((nint)i) as SFCartesianSeries;
                                if (series != null && series.XAxis != null)
                                {
                                    positon = series.XAxis.ZoomPosition;
                                    factor = 1.0f - series.XAxis.ZoomFactor;
                                    isPanEndRange = !((positon == factor && velocity.X < 0) || (positon == 0 && velocity.X > 0));
                                }
                            }
                            return isPanEndRange;
                        }
                    }
                    else
                    {
                        bool isPanEndRange = true;
                        for (int i = 0; i < formChart.Series.Count; i++)
                        {
                            SFCartesianSeries series = chart.SeriesAtIndex((nint)i) as SFCartesianSeries;
                            if (series != null && series.XAxis != null)
                            {
                                float positon = series.XAxis.ZoomPosition;
                                float factor = 1.0f - series.XAxis.ZoomFactor;
                                isPanEndRange = !((positon == factor && velocity.X < 0) || (positon == 0 && velocity.X > 0));
                            }
                        }
                        return isPanEndRange;
                    }
                }
            }

            return true;
        }
    }
}
